from typing import Any, Iterator

from ..base import BaseSolver
from ..tools import graphs


class Solver(BaseSolver):
    def print_grid(self, grid: list[tuple[int, int]], n_rows: int, n_cols: int):
        values = set(grid)
        if self.files:
            self.files.create(
                "graph.txt",
                "\n".join(
                    "".join(
                        "#" if (row, col) in values else "." for col in range(n_cols)
                    )
                    for row in range(n_rows)
                ).encode(),
                text=True,
            )
        else:
            for row in range(n_rows):
                self.logger.info(
                    "".join(
                        "#" if (row, col) in values else "." for col in range(n_cols)
                    )
                )

    def dijkstra(self, corrupted: list[tuple[int, int]], n_rows: int, n_cols: int):
        return graphs.dijkstra(
            (0, 0),
            (n_rows - 1, n_cols - 1),
            graphs.make_neighbors_grid_fn(n_rows, n_cols, set(corrupted)),
        )

    def solve(self, input: str) -> Iterator[Any]:
        values = [
            (int(p[0]), int(p[1])) for r in input.splitlines() if (p := r.split(","))
        ]

        _is_test = len(values) < 100

        n_rows, n_cols, n_bytes_p1 = (7, 7, 12) if _is_test else (71, 71, 1024)

        bytes_p1 = values[:n_bytes_p1]
        self.print_grid(bytes_p1, n_rows, n_cols)

        path_p1, cost_p1 = self.dijkstra(bytes_p1, n_rows, n_cols) or ((), -1)
        yield cost_p1

        path = path_p1
        for b in range(n_bytes_p1, len(values)):
            if values[b] not in path:
                continue
            path, _ = self.dijkstra(values[: b + 1], n_rows, n_cols) or (None, -1)
            if path is None:
                yield ",".join(map(str, values[b]))
                break
